'use strict';
const path = require('path');
const program = require('commander');
const chalk = require('chalk');
const inquirer = require('inquirer');
const tool = require('node-tool-utils');
const Logger = require('./logger');
const Action = require('./action');
const utils = require('./utils');
const builder = require('./builder');
const { EASY_CLI } = require('./constant');
global.EASY_CLI = EASY_CLI;
module.exports = class Command extends Logger {
  constructor() {
    super();
    this.baseDir = process.cwd();
    this.program = program;
    this.inquirer = inquirer;
    this.chalk = chalk;
    this.tool = tool;
    this.utils = utils;
    this.builder = builder;
    this.boilerplate = {};
    this.context = path.resolve(__dirname, '..');
    this.commands = ['init', 'install', 'dev', 'start', 'build', 'debug', 'test', 'cov', 'add', 'print', 'server', 'dll', 'zip', 'tar', 'deploy', 'clean', 'open', 'kill', 'upgrade'];
    
    this.cli = new Proxy(EASY_CLI, {
      get: function (target, key, receiver) {
        return Reflect.get(target, key, receiver);
      },
      set: function (target, key, value, receiver) {
        global.EASY_CLI[key] = value;
        return Reflect.set(target, key, value, receiver);
      }
    });
    this.action = new Action(this);
  }

  version() {
    const pkg = require(path.resolve(this.context, 'package.json'));
    this.program.version(pkg.version, '-v, --version');
  }

  option() {
    this.program
      .option('-f, --filename [path]', 'webpack config file path')
      .option('-p, --port [port]', 'webpack server port')
      .option('-t, --type [type]', 'webpack build type: client, server, web, weex')
      .option('-w, --watch', 'webpack watch and hot-update')
      .option('-m, --md5', 'webpack md5 hash js/css/image')
      .option('-c, --compress', 'webpack compress js/css/image')
      .option('-b, --build [option]', 'w(watch), m(hash) , c(compress), ex: wm/wc/mc/wmc')
      .option('-s, --size [option]', 'webpack build size analyzer tool, support size: analyzer and stats, default analyzer')
      .option('--dll', 'only webpack dll config')
      .option('--web', 'only webpack web config')
      .option('--node', 'only webpack node config')
      .option('--devtool [devtool]', 'webpack devtool config')
      .option('--webpack', 'support native webpack dev and build');
  }

  init() {
    this.program
      .command('init')
      .option('-r, --registry [url]', 'npm registry, default https://registry.npmjs.org, you can taobao registry: https://registry.npm.taobao.org')
      .option('--sync [url]', 'sync easy init prompt template config')
      .description('init webpack config or boilerplate for Vue/React/Weex')
      .action(options => {
        this.action.init(this.boilerplate, options);
      });
  }

  install() {
    this.program
      .command('install')
      .option('--mode [mode]', 'mode: npm, cnpm, tnpm, yarn and so on')
      .description('dynamic install webpack missing npm module')
      .action(options => {
        this.action.install(options);
      });
  }

  dev() {
    this.program
      .command('dev [env]', 'dev')
      .description('start webpack dev server for develoment mode')
      .action(env => {
        this.action.dev({ env });
      });
  }

  debug() {
    this.program
      .command('debug')
      .description('start project for develoment debug mode')
      .action(() => {
        this.action.debug();
      });
  }

  test() {
    this.program
      .command('test')
      .description('unit test')
      .action(() => {
        this.action.test();
      });
  }

  cov() {
    this.program
      .command('cov')
      .description('code cov')
      .action(() => {
        this.action.cov();
      });
  }

  add() {
    this.program
      .command('add')
      .description('add tempplate or component')
      .action(() => {
        this.action.add();
      });
  }

  start() {
    this.program
      .command('start [env]')
      .description('start webpack dev server for develoment mode')
      .action(env => {
        this.action.start({ env });
      });
  }

  build() {
    this.program
      .command('build [env]', 'prod')
      .option('--devtool [devtool]', 'force set webpack devtool')
      .option('--server [port]', 'start http server')
      .option('--speed', 'stat webpack build speed')
      .description('webpack building')
      .action((env = 'prod', options) => {
        this.action.build(env, options);
      });
  }

  print() {
    this.program
      .command('print [env]', 'dev')
      .option('-n, --node [key]', 'print webpack config info by config key, example: [module/module.rules/plugins] and so on(Deprecated)')
      .option('-k, --key [key]', 'print webpack config info by config key, example: [module/module.rules/plugins] and so on')
      .description('print webpack config, support print by env or config key')
      .action((env, options) => {
        this.action.print(env, options);
      });
  }

  server() {
    this.program
      .command('server')
      .option('-p, --port [port]', 'http server port')
      .option('-d, --dist [dist]', 'http server file dir')
      .option('-i, --index [index]', 'http server html index file name')
      .description('static file web http server')
      .action(options => {
        options.port = options.port || options.parent.port;
        this.action.server(options);
      });
  }

  upgrade() {
    this.program
      .command('upgrade')
      .option('--egg', 'use egg-bin and egg-scripts start application')
      .option('--babel', 'upgrade babel 6 to babel 7')
      .description('upgrade project package to latest version')
      .action(options => {
        this.action.upgrade(options);
      });
  }

  zip() {
    this.program
      .command('zip')
      .option('--filename [filename]', 'archive zip file name')
      .option('--source [path]', 'archive files root path')
      .option('--target [path]', 'archive zip file path')
      .option('--deps', 'install dependencies into node_modules')
      .option('--mode [mode]', 'mode: npm, cnpm, tnpm, yarn and so on')
      .option('--registry [registry]', 'dependence install registry url')
      .option('--nodejs', 'install node into node_modules')
      .option('--alinode', 'install alinode into node_modules')
      .description('archive files to zip file')
      .action(options => {
        this.action.zip(options);
      });
  }
  tar() {
    this.program
      .command('tar')
      .option('--filename [filename]', 'archive tar file name')
      .option('--source [path]', 'archive files root path')
      .option('--target [path]', 'archive zip file path')
      .option('--deps', 'install dependencies into node_modules')
      .option('--mode [mode]', 'mode: npm, cnpm, tnpm, yarn and so on')
      .option('--registry [registry]', 'dependence install registry url')
      .option('--node', 'install node into node_modules')
      .option('--alinode', 'install alinode into node_modules')
      .description('archive files to tar file')
      .action(options => {
        this.action.tar(options);
      });
  }

  deploy() {
    this.program
      .command('deploy')
      .description('upload file to deplay space')
      .action(options => {
        this.action.deploy(options);
      });
  }

  dll() {
    this.program
      .command('dll [env]', 'prod')
      .description('webpack dll build')
      .action(env => {
        this.action.dll(env);
      });
  }

  clean() {
    this.program
      .command('clean [dir]')
      .description('webpack cache dir clean, if dir == "all", will clean cache dir and build dir')
      .action(dir => {
        this.action.clean(dir);
      });
  }
  open() {
    this.program
      .command('open [dir]')
      .description('open webpack cache dir')
      .action(dir => {
        this.action.open(dir);
      });
  }

  kill() {
    this.program
      .command('kill [port]')
      .description('kill port process')
      .action(port => {
        this.action.kill(port);
      });
  }

  register(cmd) {
    if (this.commands.some(key => { key === cmd })) {
      this.red(`The command ${cmd} already exists. Please overwrite the command directly.`);
    } else {
      this.commands.push(cmd);
    }
  }

  command() {
    this.commands.forEach(cmd => {
      if (this[cmd]) {
        this[cmd].apply(this);
      } else {
        this.red(`The command [${cmd}] is not implemented!`);
      }
    });
  }

  parse() {
    this.program.parse(process.argv);
  }

  run() {
    this.version();
    this.option();
    this.command();
    this.parse();
  }
}